#import <Foundation/Foundation.h>

#ifdef OMP
#import <omp.h>
#endif

#import <stdio.h>
#import <stdlib.h>
#import <string.h>
#import <assert.h>
#import <math.h>

#import <vector.h>
#import <pixel.h>
#import <timer.h>

//---- prototype section ----
int main(int argc, const char *argv[]);

//---- program section ----

int main(int argc, const char *argv[])
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	pixel_t *red = [(pixel_t *)[pixel_t alloc] init: 1.0: 0.0: 0.0];
	pixel_t *grn = [(pixel_t *)[pixel_t alloc] init: 0.0: 1.0: 0.0];
	pixel_t *blu = [(pixel_t *)[pixel_t alloc] init: 0.0: 0.0: 1.0];
	pixel_t *ylw = [(pixel_t *)[pixel_t alloc] init: 1.0: 1.0: 0.0];
	pixel_t *cyn = [(pixel_t *)[pixel_t alloc] init: 0.0: 1.0: 1.0];
	pixel_t *mgn = [(pixel_t *)[pixel_t alloc] init: 1.0: 0.0: 1.0];
	pixel_t *wht = [(pixel_t *)[pixel_t alloc] init: 1.0: 1.0: 1.0];
	pixel_t *blk = [(pixel_t *)[pixel_t alloc] init: 0.0: 0.0: 0.0];

	pixel_t *pix = nil;

	int	tid,i,x,y,w=640,h=480;
	irgb_t	*imgloc,*img=NULL;

	int	chunk,ncores=1;

	timer_t	*timer = [(timer_t *)[timer_t alloc] init];

#ifdef OMP
	NSAutoreleasePool *omp_pool = nil;

  // figure out how many threads we have available
  #pragma omp parallel private(tid)
  {
    if((tid = omp_get_thread_num())==0)
      ncores = omp_get_num_threads();
  }

  // then calculate chunk, splitting up the work as evenly as possible
  chunk = h/ncores;
#endif

  // allocate 2D image array and zero out
  assert((img = (irgb_t *)malloc(w*h*sizeof(irgb_t)))!=NULL);
  memset(img,0,w*h*sizeof(irgb_t));

  [timer start];

  // set image pixel values
#ifdef OMP
#pragma omp parallel \
            shared(chunk,w,h,img) \
            private(tid,i,x,y,pix,imgloc,omp_pool)
{
  // need thread-specific AutoreleasePool
  omp_pool = [[NSAutoreleasePool alloc] init];

  pix = [(pixel_t *)[pixel_t alloc] init: 0.0: 0.0: 0.0];

  tid = omp_get_thread_num();

  #pragma omp for schedule(static,chunk)
#endif
  for(y=h-1;y>=0;y--) {
#ifndef OMP
      // quick hack to simulate 8 cores on Linux (gnustep is not thread-safe)
      tid = y/(h/8);
#endif
    for(x=0;x<w;x++) {
      [pix autorelease];
      switch(tid) {
        case 0: pix = [red copy]; break;
        case 1: pix = [grn copy]; break;
        case 2: pix = [blu copy]; break;
        case 3: pix = [ylw copy]; break;
        case 4: pix = [cyn copy]; break;
        case 5: pix = [mgn copy]; break;
        case 6: pix = [wht copy]; break;
        case 7: pix = [blk copy]; break;
      }

      // where are we in the image (using old i*c + j)
      imgloc = img + y*w + x;
      for(i=0;i<3;i++) (*imgloc)[i] = [[pix scale: 255.0] get: i];
    }
  }
#ifdef OMP
  [pix release];

  [omp_pool drain];
}
#endif

  [timer stop];
  fprintf(stderr,"cores: %d\n",ncores);
  fprintf(stderr,"time: %.2lf ms\n",[timer elapsed_ms]);

  // write out image, header first
  fprintf(stdout,"P6 %d %d 255\n",w,h);
  for(y=h-1;y>=0;y--) {
    for(x=0;x<w;x++) {
      imgloc = img + y*w + x;
      fwrite(imgloc,sizeof(irgb_t),1,stdout);
    }
  }

  // release memory
  [red release];
  [grn release];
  [blu release];
  [pix release];

  // img should really be an object
  free(img);

  [pool drain];

  return 0;
}
