// Two pixel value structures are used:
// Computations are done in the double precision domain
// where 0.0 means black and 1.0 means maximum intensity
// The raytracing algorithm can DEFINITELY produce
// pixel values that exceed 1.0 scale and then clamp
// them before assigning them to irgb_types
  
typedef double drgb_t[3];

// The .ppm file requires the usual r-g-b values in the
// range 0-255.  This structure maps one of them.
  
typedef unsigned char irgb_t[3];

//---- @interface section ----

@interface pixel_t: NSObject <NSCopying> // Class: Parent <Protocol>
{
  // instance variables
  drgb_t drgb;
}

// + class methods

// - instance methods
  // constructors (overloaded)
- (id)      init;
- (id)      init: (double) r;
- (id)      init: (double) r: (double) g;
- (id)      init: (double) r: (double) g: (double) b;

  // accessor/mutator
- (bool)    nonzero;
- (double)  get: (int) i;
- (void)    set: (int) i: (double) d;
- (void)    set: (double) r: (double) g: (double) b;

  // operators
- (pixel_t *) add: (pixel_t *) p;
- (pixel_t *) dif: (pixel_t *) p;
- (pixel_t *) scale: (double) d;

  // friends
- (void) read: (FILE *) _in;
- (void) write: (FILE *) _out: (char *) _msg;

@end
