#ifdef __APPLE__
#import <AvailabilityMacros.h>
#else
#include <math.h>
#ifndef NAN
#define NAN FP_NAN
#endif
#endif

#import <Foundation/Foundation.h>

#import <stdio.h>
#import <math.h>

#import <vector.h>

//---- @implementation section ----

@implementation vec_t

- (id) init: (double) x { return [self init: x: 0.0: 0.0]; }
- (id) init: (double) x: (double) y { return [self init: x: y: 0.0]; }
- (id) init: (double) x: (double) y: (double) z
{
  if(!(self = [super init]))
    return nil;
  vec[0] = x;  vec[1] = y;  vec[2] = z;
  return self;
}

//---- @protocols section ----
- (id) copyWithZone: (NSZone *) zone
{
  vec_t *clone = [[vec_t allocWithZone: zone] init];
  [clone set: vec[0]: vec[1]: vec[2]];
  return clone;
}

- (double) get: (int) i { return vec[i]; }
- (void) set: (int) i: (double) d { vec[i] = d; }
- (void) set: (double) x: (double) y: (double) z
{
  vec[0] = x;  vec[1] = y;  vec[2] = z;
}

- (vec_t *) plus: (vec_t *) v
{
	vec_t	*result = [[vec_t alloc] init];
	int	i;

  for(i=0;i<3;i++) [result set: i: vec[i] + [v get: i]];

  // avoid memory leak by releasing result after use (unless retain'ed)
  return [result autorelease];
}

- (vec_t *) minus: (vec_t *) v
{
	vec_t	*result = [[vec_t alloc] init];
	int	i;

  for(i=0;i<3;i++) [result set: i: vec[i] - [v get: i]];

  // avoid memory leak by releasing result after use (unless retain'ed)
  return [result autorelease];
}

- (vec_t *) scale: (double) d
{
	vec_t	*result = [[vec_t alloc] init];
	int	i;

  // avoid memory leak by releasing result after use (unless retain'ed)
  for(i=0;i<3;i++) [result set: i: vec[i] * d];

  return [result autorelease];
}

- (vec_t *) unit
{
	vec_t	*result = [[vec_t alloc] init];

  result = [[self scale: 1./[self len]] retain];

  // avoid memory leak by releasing result after use (unless retain'ed)
  return [result autorelease];
}

- (double) dot: (vec_t *) v
{
	double	s=0.0;
	int	i;

  for(i=0;i<3;i++) s += vec[i] * [v get: i];

  return s;
}

- (double) len
{
  return sqrt([self dot: self]);
}

- (int) read: (FILE *) _in
{
	int	i,c=0;

  for(i=0;i<3;i++) c += fscanf(_in,"%lf",&vec[i]);

  return (c > 0 ? c : 0);
}

- (void) write: (FILE *) _out: (char *) _msg
{	
	int	i;

  fprintf(_out,"%s",_msg);
  for(i=0; i<3; i++)
    vec[i] < 0.0 ?  fprintf(_out,"%4.0lf",vec[i]) :
                    fprintf(_out,"%2.0lf",vec[i]);
  //vec[i] < 0.0 ?  fprintf(_out,"%10.3lf",vec[i]) :
  //                fprintf(_out,"%8.3lf",vec[i]);
  fprintf(_out,"\n");
}

@end
